#include <stdlib.h>
#include <GL/glut.h>
#include <GL/gl.h>
#include "simage.h"
#include "geometry.h"

using namespace oglclass;

float viewDistance = 3.0;
float viewRotX = 0.0;
float viewRotY = 0.0;
int mode = 0;
GLuint textureID0 = 0;

// light
GLfloat lightPosition[] = {0, 2, 1, 1};
GLfloat lightAmbient[] = {0.5, 0.5, 0.5, 1.0};
GLfloat lightDiffuse[] = {1, 1, 1, 1};
GLfloat lightSpecular[] = {1, 1, 1, 0.25};

// material
GLfloat white[] = {1, 1, 1, 1};
GLfloat green[] = {0, 1, 0, 0.25};
GLfloat black[] = {0, 0, 0, 1};
GLfloat red[] = {1, 0, 0, 0.5};
GLfloat grey20[4] = {0.2, 0.2, 0.2, 1};

// sphere
geometry sph;

void initTexture(char *filename, GLuint& textureID, GLfloat wrap = GL_REPEAT)
{
	unsigned char *imageBuffer, *rescaledImage, *imagePtr;
	int imageWidth = 0, imageHeight = 0, numComponents = 0;
	imageBuffer = simage_read_image(filename, &imageWidth, &imageHeight, &numComponents);

	GLsizei xdim2,ydim2;

	xdim2 = 1;
    while (xdim2 <= imageWidth)
        xdim2 *= 2;
    xdim2 /= 2;
    ydim2 = 1;
    while (ydim2 <= imageHeight)
        ydim2 *= 2;
    ydim2 /= 2;
    if ((imageWidth != xdim2) || (imageHeight != ydim2))
        {
        rescaledImage = simage_resize(imageBuffer, imageWidth, imageHeight, numComponents,
                                           xdim2, ydim2);
        imagePtr = rescaledImage;
        }
    else
        imagePtr = imageBuffer;

	if (imagePtr != NULL)
	{
		glGenTextures(1, &textureID);
		glBindTexture(GL_TEXTURE_2D, textureID);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, numComponents == 3 ? GL_RGB : GL_RGBA, imageWidth, imageHeight,
					 0, numComponents == 3 ? GL_RGB : GL_RGBA, GL_UNSIGNED_BYTE, imageBuffer);
		glBindTexture(GL_TEXTURE_2D, 0);
	}
}

void display(void)
{
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);

	// light
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);

	// material
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, green);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, green);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50);

	// projection
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, 1, 0.1, 100);
	glMatrixMode(GL_MODELVIEW);

	// modelview
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -viewDistance);
	glRotatef(viewRotX, 1.0, 0.0, 0.0);
	glRotatef(viewRotY, 0.0, 1.0, 0.0);

	// draw sphere (LINE | FILL | TRANSPARENT | TEXTURE)
	if (mode == 0) {
		sph.setDrawStyle(GLU_LINE);
		sph.draw();
	}
	else if (mode == 1) {
		sph.setDrawStyle(GLU_FILL);
		sph.draw();
	}
	else if (mode == 2) {
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
		sph.setDrawStyle(GLU_FILL);
		sph.draw();
		glDisable(GL_BLEND);
	}
	else if (mode == 3) {
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, white);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, white);
		glBindTexture(GL_TEXTURE_2D, textureID0);
		glEnable(GL_TEXTURE_2D);
		sph.draw();
		glBindTexture(GL_TEXTURE_2D, 0);
		glDisable(GL_TEXTURE_2D);
	}

	glutSwapBuffers();
}

void key(unsigned char key, int x, int y)
{
    if (key == 27)
        exit(0);
	else if (key == ' ') {
		mode++;
		if (mode == 4) mode = 0;
	}
	else if (key == 'a') {
		viewDistance -= 1;
	}
	else if (key == 's') {
		viewDistance += 1;
	}
	glutPostRedisplay();
}

void specialkey(int key, int x, int y)
{
    if (key == GLUT_KEY_LEFT)
        viewRotY = viewRotY + 3;
    else if (key == GLUT_KEY_RIGHT)
        viewRotY = viewRotY - 3;
    else if (key == GLUT_KEY_UP)
        viewRotX = viewRotX + 3;
    else if (key == GLUT_KEY_DOWN)
        viewRotX = viewRotX - 3;
    glutPostRedisplay();
}

int main(int argc, char *argv[])
{
    glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(500,500);
    glutCreateWindow("sphere");
    glutDisplayFunc(display);
    glutKeyboardFunc(key);
    glutSpecialFunc(specialkey);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_CULL_FACE);

	initTexture("earth.jpg", textureID0);
	sph.makeSphere(1.0, 32, 24);
	sph.setUseTexture(GL_TRUE);

    glutMainLoop();
    return 0;
}

